using System;
using System.Runtime.Serialization;
using System.Threading;

using IdeaBlade.EntityModel;
using IdeaBlade.Core;

namespace DomainModel {

  /// <summary>
  /// An example of how to use the "push" feature for intermediate progress notifications.
  /// <para>
  /// This example is a very simple example of how to periodically broadcast information 
  /// to client subscribers.  The data sent to subscribers is a non-primitive type, and must
  /// therefore be both serializable and a known type.
  /// </para>
  /// </summary>
  public class LongRunningProcedure {

    private static Guid __serviceKey;
    private static INotificationManager __notificationManager;
    private static EntityManager __entityManager;

    /// <summary>
    /// Method "subscribed" to from a client in the RegisterCallback() method.  This method will start on a new
    /// thread upon the first client request, and run as long as there are subscribers or until the method exits.
    /// </summary>
    /// <param name="serviceKey">Uniquely identifies your "service".  Use this when calling the INotificationManager.</param>
    /// <param name="notificationManager">The INotificationManager you will use to request information about subscribers and send them data.</param>
    /// <param name="entityManager">A server-side EntityManager.</param>
    [AllowRpc]
    public static void MultiStepService(Guid serviceKey, INotificationManager notificationManager, EntityManager entityManager) {

      __serviceKey = serviceKey;
      __notificationManager = notificationManager;
      __entityManager = entityManager;

      TraceFns.WriteLine("LongRunningProcedure sample started");

      // Note this procedure starts only upon first subscription.  Subsequent subscribers will be notified
      // of current progress, but each doesn't start its own long running procedure.  You could do this by
      // periodically calling GetSubscribers, starting the procedure they request, and then tracking all
      // this.

      Step1();
      Step2();
      Step3();
      Step4();

      TraceFns.WriteLine("LongRunningProcedure sample exiting");
    }

    private static void Step1() {
      // Assume some logic is performed that represents 20% of the total workload.
      Thread.Sleep(1000 * 5);
      __notificationManager.Send(__serviceKey, new LongRunningInfo("The LongRunningProcedure is 20% done.", 20));
    }

    private static void Step2() {
      // Assume some logic is performed that represents 50% of the total workload.
      Thread.Sleep(1000 * 5);
      __notificationManager.Send(__serviceKey, new LongRunningInfo("The LongRunningProcedure is 50% done.", 50));
    }
    private static void Step3() {
      // Assume some logic is performed that represents 90% of the total workload.
      Thread.Sleep(1000 * 5);
      __notificationManager.Send(__serviceKey, new LongRunningInfo("The LongRunningProcedure is 90% done.", 90));
    }

    private static void Step4() {
      // Assume some logic is performed that represents 100% of the total workload.
      Thread.Sleep(1000 * 5);
      __notificationManager.Send(__serviceKey, new LongRunningInfo("The LongRunningProcedure has completed.", 100));
    }

  }

  
}
