using System;
using System.Linq;
using System.Threading;

using IdeaBlade.EntityModel;
using IdeaBlade.Core;

namespace DomainModel {

  /// <summary>
  /// An example of how to use the "push" feature.
  /// <para>
  /// This example is a very simple example of how to periodically broadcast information 
  /// to client subscribers.  This example periodically wakes up to query for new orders using
  /// the provided EntityManager, and then sends the count of new orders to all subscribed.
  /// </para>
  /// <para>
  /// This example is so simple (:-0) that you will need to provide your own means of adding orders
  /// to the OrderSummary table it watches.
  /// </para>
  /// </summary>
  public class OrderWatcher {

    private static Guid __serviceKey;
    private static INotificationManager __notificationManager;
    private static DomainModel.NorthwindIBEntities __entityManager;
    private static DateTime __lastCheckTime = DateTime.SpecifyKind(DateTime.Today, DateTimeKind.Utc);

    /// <summary>
    /// Method "subscribed" to from a client in the EM.RegisterCallback() method.  This method will start on a new
    /// thread upon the first client request, and run as long as there are subscribers or until the method exits.
    /// </summary>
    /// <param name="serviceKey">Uniquely identifies your "service".  Use this when calling the INotificationManager.</param>
    /// <param name="notificationManager">The INotificationManager you will use to request information about subscribers and send them data.</param>
    /// <param name="entityManager">A server-side EntityManager.</param>
    [AllowRpc]
    public static void NewOrderService(Guid serviceKey, INotificationManager notificationManager, EntityManager entityManager) {

      __serviceKey = serviceKey;
      __notificationManager = notificationManager;
      __entityManager = new NorthwindIBEntities(entityManager);   // Create a sub-typed server-side EM.

      TraceFns.WriteLine("OrderWatcher sample started");

      while (true) {
        CheckForNewOrders();
        // Sleep for 20 seconds between checks.
        Thread.Sleep(1000 * 20 * 1);
      }

      // This example won't currently get here (because it doesn't exit the while loop) ... the "service" ends when the method exits.
      TraceFns.WriteLine("OrderWatcher sample exiting");
    }

    private static void CheckForNewOrders() {

      var newOrders = __entityManager.Orders.Where(o => o.OrderDate >= __lastCheckTime).ToList();
      __lastCheckTime = DateTime.SpecifyKind(DateTime.Now, DateTimeKind.Utc);

      // Look at client args to see which orders they want to see - here arg0 indicates the employeeID
      if (newOrders.Count() == 0) return;

      foreach (var subscriber in __notificationManager.GetSubscribers(__serviceKey)) {
        var args = subscriber.ClientArguments;
        if (args != null && args.Count() >= 1) {
          int empId = Convert.ToInt32(args[0]);
          string msg = newOrders.Where(o => o.EmployeeID == empId).Select(o => o.OrderID).ToAggregateString(", ");
          if (!string.IsNullOrEmpty(msg)) {
            __notificationManager.Send(__serviceKey, subscriber, string.Format("New orders for employee {0}: {1}", empId, msg));
          }
        }
      }
    }

  }
}
