﻿using System.Windows;
using System.Windows.Controls;
using DomainModel;
using IdeaBlade.EntityModel;

namespace SilverlightApplication1 {

  public partial class MainPage : UserControl {

    private NorthwindIBEntities _entityManager;
    private string _owToken;

    public MainPage() {
      InitializeComponent();
      this.Loaded += new RoutedEventHandler(MainPage_Loaded);
    }

    void MainPage_Loaded(object sender, RoutedEventArgs e) {
      // When connecting to a distributed BOS, it's always a good idea
      // to set up an error handler and issue an explicit Connect.
      _entityManager = new NorthwindIBEntities(false);
      _entityManager.EntityServerError += (o, args) => {
        MessageBox.Show(args.Exception.Message);
      };
      _entityManager.ConnectAsync(op => {
        if (op.HasError) {
          MessageBox.Show(op.Error.ToString());
        } else {
          this.btnSubscribeLRP.IsEnabled = this.btnSubscribeOW.IsEnabled = true;
        }
      });
    }

    private void btnSubscribeLRP_Click(object sender, RoutedEventArgs e) {
      SubscribeLRP();
    }

    private void btnSubscribeOW_Click(object sender, RoutedEventArgs e) {
      SubscribeOW();
    }

    private void btnUnsubscribeOW_Click(object sender, RoutedEventArgs e) {
      UnsubscribeOW();
    }

    #region LongRunningProcedure
    // Subscribe to a LongRunningProcedure, which provides periodic status notifications.
    // When complete, we automatically unsubscribe from it.

    private void SubscribeLRP() {

      // Register for the service.
      _entityManager.RegisterCallback(
        "DomainModel.LongRunningProcedure, DomainModel",      /* Type name */
        "MultiStepService",                                   /* Method name */
        LongRunningProcedureNotification,                     /* the callback */
        "lrp");                                               /* user token */

      this.btnSubscribeLRP.IsEnabled = false;
      MessageBox.Show("You are now subscribed to the LongRunningProcedure.");
    }


    private void LongRunningProcedureNotification(SubscriptionOperation op) {
      if (op.HasError) {
        MessageBox.Show(op.Error.Message);
        this.btnSubscribeLRP.IsEnabled = true;
        return;
      }

      // We know this service returns one arg.
      LongRunningInfo info = op.NotificationData[0] as LongRunningInfo;
      string msg = info.Message;
      int workComplete = info.CompletionPercent;
      MessageBox.Show(msg, "Received from LongRunningProcedure", MessageBoxButton.OK);

      // We can unsubscribe once the procedure completes.
      if (workComplete == 100) {
        op.CancelCallback();
        this.btnSubscribeLRP.IsEnabled = true;
      }
    }
    #endregion

    #region OrderWatcher subscribe/unsubscribe

    // Remember to add orders (manually or through another program) to
    // have OrderWatcher broadcast messages.
    // This will do the job:
    //     insert into [Order](customerid, employeeid, orderdate, rowversion)
    //     select customerid, 1, getdate(), 1 from customer where CompanyName = 'QUICK-Stop'

    private void SubscribeOW() {
      _owToken = "2";     // Arbitrary user token - we'll use this to unsubscribe.
      int employeeId = 1;  // We'll pass this as a "client argument" into the service.

      // Register
      _entityManager.RegisterCallback(
        "DomainModel.OrderWatcher, DomainModel",        /* Type name */
        "NewOrderService",                              /* Method name */
        OrderPushCallback,                              /* the callback */
        _owToken,                                       /* user token */
        employeeId);                                    /* client argument(s) */

      // This example contains only a single subscription to this service from this client.  You
      // can subscribe to this or other services multiple times - each subscription
      // needs to use a unique UserToken.
      this.btnUnsubscribeOW.IsEnabled = true;
      this.btnSubscribeOW.IsEnabled = false;

      MessageBox.Show("You are now subscribed to the OrderWatcher.");
    }

    private void UnsubscribeOW() {
      // Cancel based on the UserToken we subscribed with.
      _entityManager.CancelCallback(_owToken);

      this.btnUnsubscribeOW.IsEnabled = false;
      this.btnSubscribeOW.IsEnabled = true;
      MessageBox.Show("You are now unsubscribed from the OrderWatcher.");
    }

    private void OrderPushCallback(SubscriptionOperation op) {
      if (op.HasError) {
        MessageBox.Show(op.Error.Message);
        this.btnUnsubscribeOW.IsEnabled = false;
        this.btnSubscribeOW.IsEnabled = true;
        return;
      }
      // We know the service is returning a single message string...
      string msg = op.NotificationData[0].ToString();
      MessageBox.Show(msg, "Received from OrderWatcher", MessageBoxButton.OK);
    }

    #endregion
  }
}
