Imports System.Threading

Imports IdeaBlade.EntityModel
Imports IdeaBlade.Core

Namespace DomainModel

  ''' <summary>
  ''' An example of how to use the "push" feature.
  ''' <para>
  ''' This example is a very simple example of how to periodically broadcast information 
  ''' to client subscribers.  This example periodically wakes up to query for new orders using
  ''' the provided EntityManager, and then sends the count of new orders to all subscribed.
  ''' </para>
  ''' <para>
  ''' This example is so simple (:-0) that you will need to provide your own means of adding orders
  ''' to the OrderSummary table it watches.
  ''' </para>
  ''' </summary>
  Public Class OrderWatcher

	Private Shared __serviceKey As Guid
	Private Shared __notificationManager As INotificationManager
        Private Shared __entityManager As NorthwindIBEntities
	Private Shared __lastCheckTime As Date = Date.SpecifyKind(Date.Today, DateTimeKind.Utc)

	''' <summary>
	''' Method "subscribed" to from a client in the EM.RegisterCallback() method.  This method will start on a new
	''' thread upon the first client request, and run as long as there are subscribers or until the method exits.
	''' </summary>
	''' <param name="serviceKey">Uniquely identifies your "service".  Use this when calling the INotificationManager.</param>
	''' <param name="notificationManager">The INotificationManager you will use to request information about subscribers and send them data.</param>
	''' <param name="entityManager">A server-side EntityManager.</param>
	<AllowRpc>
	Public Shared Sub NewOrderService(ByVal serviceKey As Guid, ByVal notificationManager As INotificationManager, ByVal entityManager As EntityManager)

	  __serviceKey = serviceKey
	  __notificationManager = notificationManager
	  __entityManager = New NorthwindIBEntities(entityManager) ' Create a sub-typed server-side EM.

	  TraceFns.WriteLine("OrderWatcher sample started")

	  Do
		CheckForNewOrders()
		' Sleep for 20 seconds between checks.
		Thread.Sleep(1000 * 20 * 1)
	  Loop

	  ' This example won't currently get here (because it doesn't exit the while loop) ... the "service" ends when the method exits.
	  TraceFns.WriteLine("OrderWatcher sample exiting")
	End Sub

	Private Shared Sub CheckForNewOrders()

	  Dim newOrders = __entityManager.Orders.Where(Function(o) o.OrderDate >= __lastCheckTime).ToList()
	  __lastCheckTime = Date.SpecifyKind(Date.Now, DateTimeKind.Utc)

	  ' Look at client args to see which orders they want to see - here arg0 indicates the employeeID
	  If newOrders.Count() = 0 Then
		  Return
	  End If

	  For Each subscriber In __notificationManager.GetSubscribers(__serviceKey)
		Dim args = subscriber.ClientArguments
		If args IsNot Nothing AndAlso args.Count() >= 1 Then
		  Dim empId As Integer = Convert.ToInt32(args(0))
		  Dim msg As String = newOrders.Where(Function(o) o.EmployeeID = empId).Select(Function(o) o.OrderID).ToAggregateString(", ")
		  If Not String.IsNullOrEmpty(msg) Then
			__notificationManager.Send(__serviceKey, subscriber, String.Format("New orders for employee {0}: {1}", empId, msg))
		  End If
		End If
	  Next subscriber
	End Sub

  End Class
End Namespace
